@@ -608,11 +608,19 @@ class CxxParser:
                         param = self._parse_template_type_parameter(tok, None)
                     else:
                         param, _ = self._parse_parameter(
-                            ptok, TemplateNonTypeParam, False, ">"
+                            ptok,
+                            TemplateNonTypeParam,
+                            concept_ok=False,
+                            deduce_this_ok=False,
+                            end=">",
                         )
                 else:
                     param, _ = self._parse_parameter(
-                        tok, TemplateNonTypeParam, concept_ok=False, end=">"
+                        tok,
+                        TemplateNonTypeParam,
+                        concept_ok=False,
+                        deduce_this_ok=False,
+                        end=">",
                     )
 
                 params.append(param)
@@ -718,6 +726,13 @@ class CxxParser:
             if dtype:
                 args.append(TemplateArgument(dtype, param_pack))
             else:
+                # special case for sizeof...(thing)
+                if param_pack and val.tokens[-1].value == "sizeof":
+                    val.tokens.append(Token("...", "ELLIPSIS"))
+                    tok = self._next_token_must_be("(")
+                    raw_toks = self._consume_balanced_tokens(tok)
+                    val.tokens.extend(Token(tok.value, tok.type) for tok in raw_toks)
+
                 args.append(TemplateArgument(val, param_pack))
 
             tok = self._next_token_must_be(",", ">")
@@ -894,11 +909,17 @@ class CxxParser:
             raise CxxParseError("internal error")
 
     def _consume_gcc_attribute(
-        self, tok: LexToken, doxygen: typing.Optional[str] = None
+        self, tok: typing.Optional[LexToken], doxygen: typing.Optional[str] = None
     ) -> None:
-        tok1 = self._next_token_must_be("(")
-        tok2 = self._next_token_must_be("(")
-        self._consume_balanced_tokens(tok1, tok2)
+        while True:
+            tok1 = self._next_token_must_be("(")
+            tok2 = self._next_token_must_be("(")
+            self._consume_balanced_tokens(tok1, tok2)
+
+            # Apparently you can have multiple attributes chained together?
+            tok = self.lex.token_if("__attribute__")
+            if tok is None:
+                break
 
     def _consume_declspec(
         self, tok: LexToken, doxygen: typing.Optional[str] = None
@@ -1041,7 +1062,9 @@ class CxxParser:
     ) -> None:
         self.state.location = tok.location
 
-        tok = self._next_token_must_be("NAME", "DBL_COLON", "namespace", "typename")
+        tok = self._next_token_must_be(
+            "NAME", "DBL_COLON", "namespace", "typename", "enum"
+        )
 
         if tok.type == "namespace":
             if template:
@@ -1746,6 +1769,7 @@ class CxxParser:
         tok: typing.Optional[LexToken],
         cls: typing.Type[PT],
         concept_ok: bool,
+        deduce_this_ok: bool,
         end: str = ")",
     ) -> typing.Tuple[PT, typing.Optional[Type]]:
         """
@@ -1760,6 +1784,10 @@ class CxxParser:
         param_pack = False
         parsed_type: typing.Optional[Type]
         at_type: typing.Optional[Type] = None
+        extras: typing.Dict[str, typing.Any] = {}
+
+        if deduce_this_ok and self.lex.token_if("this"):
+            extras["deduces_this"] = True
 
         if not tok:
             tok = self.lex.token()
@@ -1810,12 +1838,18 @@ class CxxParser:
         if at_type and self.lex.token_if("ELLIPSIS"):
             param_pack = True
 
-        param = cls(type=dtype, name=param_name, default=default, param_pack=param_pack)
+        param = cls(
+            type=dtype,
+            name=param_name,
+            default=default,
+            param_pack=param_pack,
+            **extras,
+        )
         self.debug_print("parameter: %s", param)
         return param, at_type
 
     def _parse_parameters(
-        self, concept_ok: bool
+        self, concept_ok: bool, deduce_this_ok: bool
     ) -> typing.Tuple[typing.List[Parameter], bool, typing.List[TemplateParam]]:
         """
         Consumes function parameters and returns them, and vararg if found, and
@@ -1839,7 +1873,13 @@ class CxxParser:
                 self._next_token_must_be(")")
                 break
 
-            param, at_type = self._parse_parameter(None, Parameter, concept_ok)
+            # Deduce-this only applicable for first function parameter
+            param, at_type = self._parse_parameter(
+                None,
+                Parameter,
+                concept_ok,
+                deduce_this_ok=deduce_this_ok and len(params) == 0,
+            )
             params.append(param)
             if at_type:
                 at_params.append(
@@ -1924,6 +1964,11 @@ class CxxParser:
         if self.lex.token_if("{"):
             self._discard_contents("{", "}")
             fn.has_body = True
+        elif self.lex.token_if("="):
+            if self.lex.token_if_val("delete"):
+                fn.deleted = True
+            else:
+                raise self._parse_error(None, "expected 'delete")
 
     def _parse_method_end(self, method: Method) -> None:
         """
@@ -1973,7 +2018,7 @@ class CxxParser:
                 if self.lex.token_if("{"):
                     self._discard_contents("{", "}")
                     method.has_body = True
-                break
+                    break
             elif tok_value == "throw":
                 tok = self._next_token_must_be("(")
                 method.throw = self._create_value(self._consume_balanced_tokens(tok))
@@ -2027,7 +2072,9 @@ class CxxParser:
         state.location = location
         is_class_block = isinstance(state, ClassBlockState)
 
-        params, vararg, at_params = self._parse_parameters(True)
+        params, vararg, at_params = self._parse_parameters(
+            True, deduce_this_ok=is_class_block
+        )
 
         # Promote abbreviated template parameters
         if at_params:
@@ -2223,7 +2270,7 @@ class CxxParser:
                     toks = self._consume_balanced_tokens(gtok)
                     self.lex.return_tokens(toks[1:-1])
 
-                fn_params, vararg, _ = self._parse_parameters(False)
+                fn_params, vararg, _ = self._parse_parameters(False, False)
 
                 assert not isinstance(dtype, FunctionType)
                 dtype = dtype_fn = FunctionType(dtype, fn_params, vararg)
@@ -2253,7 +2300,7 @@ class CxxParser:
                         assert not isinstance(dtype, FunctionType)
                         dtype = self._parse_array_type(aptok, dtype)
                     elif aptok.type == "(":
-                        fn_params, vararg, _ = self._parse_parameters(False)
+                        fn_params, vararg, _ = self._parse_parameters(False, False)
                         # the type we already have is the return type of the function pointer
 
                         assert not isinstance(dtype, FunctionType)
@@ -2369,6 +2416,8 @@ class CxxParser:
                 volatile = True
             elif tok_type in _attribute_start:
                 self._consume_attribute(tok)
+            elif tok_type in ("__inline", "__forceinline"):
+                both["inline"] = tok
             else:
                 break
 
@@ -2620,6 +2669,16 @@ class CxxParser:
         ):
             return
 
+        # Check for an abbreviated template return type, promote it
+        if not is_typedef and parsed_type is not None and self.lex.token_if_val("auto"):
+            abv_ret_tmpl = TemplateNonTypeParam(type=parsed_type, param_idx=-1)
+            if template is None:
+                template = TemplateDecl(params=[abv_ret_tmpl])
+            elif isinstance(template, TemplateDecl):
+                template.params.append(abv_ret_tmpl)
+            else:
+                template[-1].params.append(abv_ret_tmpl)
+
         var_ok = True
 
         if is_typedef:
@@ -2650,6 +2709,11 @@ class CxxParser:
                 # if it returns True then it handled the end of the statement
                 break
 
+            # Sometimes attributes come after a function
+            atok = self.lex.token_if("__attribute__")
+            if atok:
+                self._consume_gcc_attribute(atok)
+
             # Unset the doxygen, location
             doxygen = None
 
