@@ -458,13 +458,18 @@ class TemplateNonTypeParam:
        // abbreviated template parameters are converted to this and param_idx is set
        void fn(C auto p)
                ~~~~~~
+
+       // abbreviated template parameters that are return types have param_idx = -1
+       C auto fn()
+       ~~~~~~
     """
 
     type: DecoratedType
     name: typing.Optional[str] = None
     default: typing.Optional[Value] = None
 
-    #: If this was promoted, the parameter index that this corresponds with
+    #: If this was promoted, the parameter index that this corresponds with. Return
+    #: types are set to -1
     param_idx: typing.Optional[int] = None
 
     #: Contains a ``...``
@@ -660,6 +665,9 @@ class Parameter:
     default: typing.Optional[Value] = None
     param_pack: bool = False
 
+    #: Indicates that this parameter is the parameter used for cxx23 deducing-this
+    deduces_this: bool = False
+
     def format(self) -> str:
         default = f" = {self.default.format()}" if self.default else ""
         pp = "... " if self.param_pack else ""
@@ -691,6 +699,7 @@ class Function:
     extern: typing.Union[bool, str] = False
     static: bool = False
     inline: bool = False
+    deleted: bool = False
 
     #: If true, the body of the function is present
     has_body: bool = False
@@ -759,7 +768,6 @@ class Method(Function):
     constructor: bool = False
     explicit: bool = False
     default: bool = False
-    deleted: bool = False
 
     destructor: bool = False
 
